@@ -18,8 +18,8 @@
 # -*- coding: utf-8 -*-
 import os
 import re
+import sorl
 
-import django
 from django.conf import settings
 if settings.USE_GIS:
     from django.contrib.gis.db import models
@@ -28,10 +28,8 @@ else:
 from django.contrib.auth.models import User
 from django.contrib.contenttypes.models import ContentType
 from django.core.management import call_command
-from django.core.exceptions import ObjectDoesNotExist, SuspiciousOperation
+from django.core.exceptions import ObjectDoesNotExist, SuspiciousOperation, ValidationError
 from django.db.models import signals, permalink
-from django.db.models.fields.related import OneToOneRel
-from django.db.models.query import delete_objects, CollectedObjects
 from django.db.models.signals import post_save
 from django.template.loader import render_to_string
 from django.utils.encoding import force_unicode
@@ -40,7 +38,6 @@ from django.utils.text import unescape_entities
 from django.utils.translation import ugettext_lazy as _
 from django.utils.translation import ugettext
 
-from cmsutils.db.fields import AutoSlugField, JSONField
 from cmsutils.signals import post_rebuild_db
 if settings.USE_GIS:
     from south.introspection_plugins import geodjango  # pyflakes:ignore
@@ -51,11 +48,13 @@ from transmeta import TransMeta, get_fallback_fieldname
 from tagging.models import Tag
 from tagging.fields import TagField
 
+from merengue.base.dbfields import AutoSlugField, JSONField
 from merengue.base.managers import BaseContentManager, WorkflowManager
 from merengue.base.review_tasks import review_to_pending_status
 from merengue.urlresolvers import get_url_default_lang
 from merengue.multimedia.models import BaseMultimedia
 from merengue.utils import is_last_application
+from merengue.workflow.models import State
 
 
 PRIORITY_CHOICES = (
@@ -69,7 +68,6 @@ PRIORITY_CHOICES = (
     (8, _('Highest')),
     )
 
-
 COMMENTABLE_CHOICES = (
     ('disabled', _('disabled')),
     ('allowed', _('allowed')),
@@ -77,6 +75,9 @@ COMMENTABLE_CHOICES = (
 
 
 class ContactInfo(models.Model):
+    """
+    Store the contact information of all managed contents
+    """
     name = models.CharField(verbose_name=_('name'), max_length=200,
                             blank=True, null=True)
     contact_email = models.EmailField(verbose_name=_('contact email'),
@@ -99,8 +100,15 @@ class ContactInfo(models.Model):
     def __unicode__(self):
         return self.contact_email or ugettext('Without contact email')
 
+    def is_empty(self):
+        return not (self.name or self.contact_email or self.contact_email2 or self.url or \
+                    self.phone or self.phone2 or self.fax)
+
 
 class BaseCategory(models.Model):
+    """
+    Abstract model for all models kind of categories
+    """
     __metaclass__ = TransMeta
     name = models.CharField(verbose_name=_('name'), max_length=200)
     slug = models.SlugField(verbose_name=_('slug'), max_length=200, db_index=True)
@@ -116,22 +124,28 @@ class BaseCategory(models.Model):
 
     @permalink
     def get_admin_absolute_url(self):
+        """ Link to the admin page for editing the object """
         content_type = ContentType.objects.get_for_model(self)
         return ('merengue.base.views.admin_link', [content_type.id, self.id, ''])
 
 
 class Base(models.Model):
+    """
+    Abstract model for models with default management features
+    """
     __metaclass__ = TransMeta
     name = models.CharField(verbose_name=_('name'), max_length=200, db_index=True)
-    slug = models.SlugField(verbose_name=_('slug'), max_length=200, db_index=True, unique=True)
+    slug = models.SlugField(verbose_name=_('slug'), max_length=200, db_index=True)
     plain_description = models.TextField(verbose_name=_('plain text description'),
                                          null=True, blank=True, editable=False)
     description = models.TextField(verbose_name=_('description'),
                                    null=True, blank=True)
 
     status = models.CharField(_('Publication status'), max_length=20, choices=settings.STATUS_LIST,
-                              default='draft', help_text=_('Enter the current status'), db_index=True,
+                              help_text=_('Enter the current status'), db_index=True,
                               editable=True)
+    workflow_status = models.ForeignKey(State, verbose_name=_('workflow status'),
+                                       null=True, blank=True, editable=True)
     main_image = StdImageField(_('main image'), upload_to='content_images',
                                thumbnail_size=(200, 200),
                                null=True, blank=True, editable=False)
@@ -158,7 +172,30 @@ class Base(models.Model):
         else:
             setattr(self, to_field, original_text)
 
+    def clean(self):
+        """ Override the Django one to take into account integer overflows """
+        super(Base, self).clean()
+        numeric_fields = [models.IntegerField, models.PositiveIntegerField,
+                          models.PositiveSmallIntegerField, models.SmallIntegerField]
+        invalid_fields = set()
+        for field in self._meta.fields:
+            for numeric in numeric_fields:
+                if isinstance(field, numeric) and field.value_from_object(self) > settings.MAX_INT_VALUE:
+                    invalid_fields.add(field)
+                    continue
+        if invalid_fields:
+            error_msg = u"%s: %s %d" % (', '.join([force_unicode(f.verbose_name) for f in invalid_fields]),
+                                        _('Numeric fields can not be larger than'),
+                                        settings.MAX_INT_VALUE)
+            raise ValidationError(error_msg)
+
+    def get_status_object(self):
+        # since the connection with the state is created on the post-save
+        # signal, it will always exists.
+        return self.stateobjectrelation_set.get().state
+
     def save(self, *args, **kwargs):
+        """ Override Django one to generate a plain text representation """
         for lang in settings.LANGUAGES:
             field_name = 'description_%s' % lang[0]
             to_field = 'plain_description_%s' % lang[0]
@@ -166,8 +203,40 @@ class Base(models.Model):
 
         super(Base, self).save(*args, **kwargs)
 
+    def populate_workflow_status(self, force_update=False, raw=False):
+        """ Populates the workflow status from the status slug """
+        from merengue.workflow.utils import workflow_by_model
+        workflow_status = getattr(self, 'workflow_status', None)
+        if not workflow_status:
+            workflow = workflow_by_model(self.__class__)
+            self.workflow_status = workflow.get_initial_state()
+            workflow_status = self.workflow_status
+        if force_update or (getattr(self, 'status', None) is not None and
+           workflow_status and self.status != self.workflow_status.slug):
+            self.update_status(raw)
+
+    def update_status(self, raw=False):
+        """ Updates the status object and updates the permissions.
+            If raw=True we will use save_base(raw) to avoid emit any signal """
+        from merengue.perms.models import ObjectPermission
+        # an extra check to avoid possibles infinte recursion
+        if self.status != self.workflow_status.slug:
+            self.status = self.workflow_status.slug
+            if hasattr(self, 'objectpermission_set'):
+                self.objectpermission_set.all().delete()
+                for perm in self.workflow_status.statepermissionrelation_set.all():
+                    ObjectPermission.objects.get_or_create(content=self,
+                                                    role=perm.role,
+                                                    permission=perm.permission)
+            if raw:
+                # this avoid to emit signals. Useful when loading from fixtures
+                self.save_base(raw=True)
+            else:
+                self.save()
+
     @permalink
     def get_admin_absolute_url(self):
+        """ Link to the admin page for editing the object """
         content_type = ContentType.objects.get_for_model(self)
         return ('merengue.base.views.admin_link', [content_type.id, self.id, ''])
 
@@ -185,7 +254,8 @@ if settings.USE_GIS:
         map_icon = StdImageField(_('map icon'), upload_to='map_icons',
                                    null=True, blank=True)
         location = models.ForeignKey(Location, verbose_name=_('location'),
-                                     null=True, blank=True, editable=False)
+                                     null=True, blank=True, editable=False,
+                                     on_delete=models.SET_NULL)
         location.delete_cascade = False
 
         objects = WorkflowManager()
@@ -276,20 +346,35 @@ class BaseContentMeta(TransMeta):
             delattr(attrs['Meta'], 'content_view_function')
         else:
             content_view_function = None
+        if 'Meta' in attrs and hasattr(attrs['Meta'], 'check_slug_uniqueness'):
+            check_slug_uniqueness = attrs['Meta'].check_slug_uniqueness
+            delattr(attrs['Meta'], 'check_slug_uniqueness')
+        else:
+            check_slug_uniqueness = False
 
         new_class = super(BaseContentMeta, cls).__new__(cls, name, bases, attrs)
         if hasattr(new_class, '_meta'):
             new_class._meta.content_view_template = content_view_template
             new_class._meta.content_view_function = content_view_function
+            new_class._meta.check_slug_uniqueness = check_slug_uniqueness
         return new_class
 
 
 class BaseContent(BaseClass):
+    """
+    Merengue managed content types use relational database inheritance to have
+    a non abstract base managed model to be able to selecting all objects with
+    only one SQL.
+
+    If you want to create a new content type you should inherits from this model.
+    """
     __metaclass__ = BaseContentMeta
 
     contact_info = models.ForeignKey(ContactInfo,
                                      verbose_name=_('contact info'),
-                                     null=True, blank=True, editable=False)
+                                     null=True, blank=True, editable=False,
+                                     on_delete=models.SET_NULL)
+    contact_info.delete_cascade = False
     related_items = models.ManyToManyField('BaseContent',
                                            verbose_name=_('related items'),
                                            null=True, blank=True, editable=False)
@@ -306,29 +391,29 @@ class BaseContent(BaseClass):
                                    null=True, blank=True, editable=False)
 
     last_editor = models.ForeignKey(User, null=True, blank=True, editable=False,
-                                    related_name='last_edited_content')
-    last_editor.delete_cascade = False
+                                    related_name='last_edited_content',
+                                    verbose_name=_('last editor'),
+                                    on_delete=models.SET_NULL)
 
     # permission global
     adquire_global_permissions = models.BooleanField(_('Adquire global permissions'), default=True)
 
     # tagging info
     tags = TagField(verbose_name=_('Tags'),
-                    help_text=_('Tags will be splitted by spaces or commas if present. To force \
-                                tag names, use double quotes.'))
+                    help_text=_('Tags will be splitted by commas.'))
 
     # meta info
-    meta_desc = models.TextField(verbose_name=_('meta description'), null=True, blank=True)
+    meta_desc = models.TextField(verbose_name=_('meta description'),
+                                 null=True, blank=True)
 
-    commentable = models.CharField(_('comments'), max_length=20, choices=COMMENTABLE_CHOICES,
-                              default='allowed', help_text=_('Is that content commentable'),
-                              editable=True)
+    commentable = models.CharField(_('comments'), max_length=20, default='allowed',
+                                   choices=COMMENTABLE_CHOICES, editable=True,
+                                   help_text=_('Is that content commentable'))
 
     # multimedia resources
-    multimedia = models.ManyToManyField(BaseMultimedia,
+    multimedia = models.ManyToManyField(BaseMultimedia, blank=True,
                                         verbose_name=_('multimedia'),
-                                        through='MultimediaRelation',
-                                        blank=True)
+                                        through='MultimediaRelation')
 
     # cached class name from this content
     # this should has null=False, blank=False but in practice this
@@ -350,6 +435,15 @@ class BaseContent(BaseClass):
                                     null=True, blank=True,
                                     related_name='contents_owned')
 
+    participants = models.ManyToManyField(User,
+                                          verbose_name=_('participants'),
+                                          null=True, blank=True,
+                                          related_name='contents_participated')
+
+    position = models.PositiveIntegerField(verbose_name=_('position'),
+                                           null=True,
+                                           editable=False)
+
     # structural contents
     no_changeable = models.BooleanField(default=False,
                                         editable=False)
@@ -390,17 +484,25 @@ class BaseContent(BaseClass):
         translate = ('cached_plain_text', )
         abstract = False
         #content_view_template = 'content_view.html' # default definition by BaseContentMeta metaclass
-        ordering = (get_fallback_fieldname('name'), )
+        ordering = ('position', get_fallback_fieldname('name'), )
+        check_slug_uniqueness = True
 
-    def admin_absolute_url(self):
-        return '<a href="%s">%s</a>' % (self.get_admin_absolute_url(), self.name)
-    admin_absolute_url.allow_tags = True
+    def admin_link_markup(self):
+        return '<a href="%s">%s</a>' % (self.get_real_instance().get_admin_absolute_url(), self.name)
+    admin_link_markup.allow_tags = True
+    admin_link_markup.short_description = _('Name')
 
-    @classmethod
-    def get_menu_name(cls):
-        return u"%s_menu" % cls._meta.module_name
+    def save(self, update_rank=False, **kwargs):
+        """ Do extra logic like setting ordering, update ranking if needed remove some tags """
+        # new objects should be added in last place
+        if not self.id:
+            try:
+                ordered = self.__class__.objects.filter(position__isnull=False).order_by('-position')
+                last = ordered[0]
+                self.position = last.position + 1
+            except IndexError:
+                pass
 
-    def save(self, update_rank=True, **kwargs):
         super(BaseContent, self).save(**kwargs)
         object_update_again = False
 
@@ -431,21 +533,45 @@ class BaseContent(BaseClass):
                 # we force an update since we already did an insert
                 super(BaseContent, self).save(force_update=True)
 
+    def validate_unique(self, exclude=None):
+        """ Check the slug uniqueness """
+        errors = {}
+        try:
+            super(BaseContent, self).validate_unique(exclude)
+        except ValidationError, validation_errors:
+            errors = validation_errors.update_error_dict(errors)
+        if self._meta.check_slug_uniqueness:
+            # validate that slug is unique in the model
+            content_with_same_slug = self.__class__.objects.filter(slug=self.slug).exclude(pk=self.pk).exists()
+            if content_with_same_slug:
+                errors.setdefault('slug', []).append(ugettext(u'Please set other slug. This slug has been assigned'))
+        if errors:
+            raise ValidationError(errors)
+
+    @classmethod
+    def get_subclasses(cls):
+        subclasses = cls.__subclasses__()
+        result = []
+        active_models = models.get_models()
+        for subclass in subclasses:
+            if subclass not in active_models:
+                continue
+            if not subclass._meta.abstract:
+                result.append(subclass)
+            result += subclass.get_subclasses()
+        return result
+
     def get_real_instance(self):
-        """ get object child instance """
-        def get_subclasses(cls):
-            subclasses = cls.__subclasses__()
-            result = []
-            for subclass in subclasses:
-                if not subclass._meta.abstract:
-                    result.append(subclass)
-                else:
-                    result += get_subclasses(subclass)
-            return result
+        """
+        BaseContent objects are only "abstract" managed contents.
+        get_real_instance returns the real object: news item, event, etc.
+
+        Makes a SQL sentence which does the JOIN with its real model class
+        """
 
         if hasattr(self, '_real_instance'):  # try looking in our cache
             return self._real_instance
-        subclasses = get_subclasses(self.__class__)
+        subclasses = self.__class__.get_subclasses()
         if not subclasses:  # already real_instance
             real_instance = getattr(self, self.class_name, self)
             self._real_instance = real_instance
@@ -454,36 +580,62 @@ class BaseContent(BaseClass):
             subclasses_names = [cls.__name__.lower() for cls in subclasses]
             for subcls_name in subclasses_names:
                 if hasattr(self, subcls_name):
-                    return getattr(self, subcls_name, self).get_real_instance()
+                    self._real_instance = getattr(self, subcls_name, self).get_real_instance()
+                    return self._real_instance
+            self._real_instance = self
             return self
 
+    def get_parent_for_permissions(self):
+        if settings.ACQUIRE_SECTION_ROLES:
+            return self.get_main_section()
+        return None
+
     @permalink
     def get_absolute_url(self):
         return ('merengue.base.views.public_link', [self._meta.app_label, self._meta.module_name, self.id])
 
-    @permalink
-    def public_link_without_section(self):
-        return self._public_link_without_section()
-
-    def _public_link_without_section(self):
-        return ('merengue.base.views.public_view', [self._meta.app_label, self._meta.module_name, self.id, self.slug])
-
     @permalink
     def public_link(self):
+        """ Get the content public link, depending on if is inside a section or not """
         section = self.get_main_section()
         if section:
             return section.get_real_instance()._content_public_link(section, self)
         else:
             return self._public_link_without_section()
 
+    @permalink
+    def public_link_without_section(self):
+        """ Get the content public link without taking into account its section """
+        return self._public_link_without_section()
+
+    def _public_link_without_section(self):
+        return ('merengue.base.views.public_view', [self._meta.app_label, self._meta.module_name, self.id, self.slug])
+
     def link_by_user(self, user):
-        """ User dependent link. To override in subclasses, if needed """
+        """ Get link depending on user. To override in subclasses, if needed """
         raise NotImplementedError("Model %s has no implements a link_by_user method" % self._meta)
 
+    def get_owners(self):
+        return self.owners.all()
+
+    def get_participants(self):
+        return self.participants.all()
+
+    def can_view(self, user):
+        """ Returns if the user can edit this content """
+        from merengue.perms.utils import has_permission
+        return has_permission(self, user, 'view')
+
     def can_edit(self, user):
+        """ Returns if the user can edit this content """
         from merengue.perms.utils import has_permission
         return has_permission(self, user, 'edit')
 
+    def can_delete(self, user):
+        """ Returns if the user can delete this content """
+        from merengue.perms.utils import has_permission
+        return has_permission(self, user, 'delete')
+
     def get_main_section(self):
         """ Get main section of a content """
         try:
@@ -491,23 +643,26 @@ class BaseContent(BaseClass):
         except ObjectDoesNotExist:
             return None
 
-    def calculate_rank(self):
-        return 100.0  # default implementation
-
     def recalculate_main_image(self):
-        """ main image will be first ordered multimedia relation with class "photo" """
+        """ Main image will be first ordered multimedia relation which is a Photo instance """
         ordered_photo_relations = MultimediaRelation.objects.filter(
             multimedia__class_name='photo',
+            multimedia__status='published',
             content=self,
         ).order_by('order')
         if ordered_photo_relations:
             first_photo = ordered_photo_relations[0].multimedia.get_real_instance()
             if os.path.exists(first_photo.image.path):
+                if self.main_image:
+                    sorl.thumbnail.delete(self.main_image, delete_file=False)
+                    sorl.thumbnail.delete(self.main_image.thumbnail, delete_file=False)
                 self.main_image.save(os.path.basename(first_photo.image.name),
                                      first_photo.image)
                 self.save()
         elif self.main_image:
             # we delete photo if not exists ordered photos for this content
+            sorl.thumbnail.delete(self.main_image, delete_file=False)
+            sorl.thumbnail.delete(self.main_image.thumbnail, delete_file=False)
             self.main_image.delete()
 
     @classmethod
@@ -549,33 +704,21 @@ class BaseContent(BaseClass):
         urls = self.breadcrumbs_items()
         return render_to_string('base/breadcrumbs.html', {'urls': urls})
 
+    def update_status(self, raw=True):
+        """We assume that State is changed
+        """
+        super(BaseContent, self).update_status(raw)
+        from merengue.perms.models import ObjectPermission
+        self.objectpermission_set.all().delete()
+        for perm in self.workflow_status.statepermissionrelation_set.all():
+            ObjectPermission.objects.get_or_create(
+                content=self,
+                role=perm.role,
+                permission=perm.permission,
+            )
 
-def calculate_class_name(instance):
-    instance.class_name = instance._meta.module_name
-
-
-def base_content_pre_save_handler(sender, instance, **kwargs):
-    if isinstance(instance, BaseContent) and not instance.id:
-        calculate_class_name(instance)
-
-
-signals.pre_save.connect(base_content_pre_save_handler)
-
-
-def notify_status_changes(sender, instance, **kwargs):
-    if isinstance(instance, BaseContent):
-        if not hasattr(instance, '_original_status'):
-            return
-        if instance._original_status == instance.status:
-            return
-        if instance.status == 'pending':
-            review_to_pending_status(instance, instance._original_status)
-        # Change cached original status so if we have
-        # multiple saves we don't generate multiple
-        # review tasks
-        instance._original_status = instance.status
-
-signals.post_save.connect(notify_status_changes)
+    def calculate_rank(self):
+        return 100.0  # default implementation
 
 
 class MultimediaRelation(models.Model):
@@ -604,7 +747,7 @@ class MultimediaRelation(models.Model):
         if update_order:
             self.order = self._get_next_order()
             self.save()
-        if self.multimedia.class_name == 'photo' and self.order == 0:
+        if self.multimedia.class_name == 'photo':
             self.content.recalculate_main_image()
 
     def get_image(self):
@@ -655,138 +798,36 @@ class MultimediaRelation(models.Model):
             self.content.save()
 
 
-def ensure_break_relations(sender, instance, **kwargs):
-    if hasattr(instance, 'break_relations'):
-        instance.break_relations()
-
-signals.pre_delete.connect(ensure_break_relations)
-
-
 def init_gis(sender, **kwargs):
     call_command('init_gis')
 
 
 def recalculate_main_image(sender, instance, **kwargs):
     """ recalculate main_image after deleting a relation """
-    if instance.multimedia.class_name == 'photo' and instance.order == 0:
+    if instance.multimedia.class_name == 'photo':
         ordered_photo_relations = MultimediaRelation.objects.filter(
             multimedia__class_name='photo',
+            multimedia__status='published',
             content=instance.content,
-        ).order_by('order')
+        ).exclude(pk=instance.pk).order_by('order')
         # we can assert instance will not be in ordered_photo_relations
         if ordered_photo_relations:
             # reorder photos
             for i, mr in enumerate(ordered_photo_relations):
                 mr.order = i
                 mr.save()
-        instance.content.recalculate_main_image()
+            instance.content.recalculate_main_image()
+        elif instance.content.main_image:
+            sorl.thumbnail.delete(instance.content.main_image, delete_file=False)
+            sorl.thumbnail.delete(instance.content.main_image.thumbnail, delete_file=False)
+            instance.content.main_image.delete()
 
 
-signals.post_delete.connect(recalculate_main_image, sender=MultimediaRelation)
+signals.pre_delete.connect(recalculate_main_image, sender=MultimediaRelation)
 if settings.USE_GIS:
     post_rebuild_db.connect(init_gis)
 
 
-def _collect_sub_objects(self, seen_objs, parent=None, nullable=False):
-    """
-    Recursively populates seen_objs with all objects related to this
-    object.
-
-    When done, seen_objs.items() will be in the format:
-        [(model_class, {pk_val: obj, pk_val: obj, ...}),
-            (model_class, {pk_val: obj, pk_val: obj, ...}), ...]
-
-    Patched Django version to allowing disable delete cascade behavior in
-    foreign keys fields.
-    """
-    pk_val = self._get_pk_val()
-    # This Django version checking is a needed workaround
-    # because internal API was changed in
-    # http://code.djangoproject.com/changeset/12600
-    if django.VERSION[:3] in [(1, 1, 0), (1, 1, 1)]:
-        if seen_objs.add(self.__class__, pk_val, self,
-                         parent, nullable):
-            return
-    else:  # For Django 1.1.2 or newer
-        if seen_objs.add(self.__class__, pk_val, self,
-                         type(parent), parent, nullable):
-            return
-
-    for related in self._meta.get_all_related_objects():
-        rel_opts_name = related.get_accessor_name()
-        delete_cascade = getattr(related.field, 'delete_cascade', True)
-        if not delete_cascade:
-            continue
-        if isinstance(related.field.rel, OneToOneRel):
-            try:
-                sub_obj = getattr(self, rel_opts_name)
-            except ObjectDoesNotExist:
-                pass
-            else:
-                sub_obj._collect_sub_objects(seen_objs, self, related.field.null)
-        else:
-            # To make sure we can access all elements, we can't use the
-            # normal manager on the related object. So we work directly
-            # with the descriptor object.
-            for cls in self.__class__.mro():
-                if rel_opts_name in cls.__dict__:
-                    rel_descriptor = cls.__dict__[rel_opts_name]
-                    break
-            else:
-                raise AssertionError("Should never get here.")
-            delete_qs = rel_descriptor.delete_manager(self).all()
-            for sub_obj in delete_qs:
-                sub_obj._collect_sub_objects(seen_objs, self, related.field.null)
-
-    # Handle any ancestors (for the model-inheritance case). We do this by
-    # traversing to the most remote parent classes -- those with no parents
-    # themselves -- and then adding those instances to the collection. That
-    # will include all the child instances down to "self".
-    parent_stack = [p for p in self._meta.parents.values() if p is not None]
-    while parent_stack:
-        link = parent_stack.pop()
-        parent_obj = getattr(self, link.name)
-        if parent_obj._meta.parents:
-            parent_stack.extend(parent_obj._meta.parents.values())
-            continue
-        # At this point, parent_obj is base class (no ancestor models). So
-        # delete it and all its descendents.
-        parent_obj._collect_sub_objects(seen_objs)
-
-
-def break_relations(self):
-    for related in self._meta.get_all_related_objects():
-        rel_opts_name = related.get_accessor_name()
-        field = related.field
-        delete_cascade = getattr(field, 'delete_cascade', True)
-        if not delete_cascade:
-            if isinstance(related.field.rel, models.OneToOneRel):
-                setattr(self, rel_opts_name, None)
-            else:
-                for sub_obj in getattr(self, rel_opts_name).all():
-                    setattr(sub_obj, related.field.name, None)
-                    sub_obj.save()
-
-
-def delete(self):
-    assert self._get_pk_val() is not None, "%s object can't be deleted because its %s attribute is set to None." % (self._meta.object_name, self._meta.pk.attname)
-
-    # Break relations
-    self.break_relations()
-
-    # Find all the objects than need to be deleted.
-    seen_objs = CollectedObjects()
-    self._collect_sub_objects(seen_objs)
-
-    # Actually delete the objects.
-    delete_objects(seen_objs)
-
-delete.alters_data = True
-
-models.Model.add_to_class('delete', delete)
-models.Model.add_to_class('break_relations', break_relations)
-models.Model.add_to_class('_collect_sub_objects', _collect_sub_objects)
-
 # ----- adding south rules to help introspection -----
 
 rules = [
@@ -799,23 +840,24 @@ rules = [
 
 add_introspection_rules(rules, ["^cmsutils\.db\.fields\.AutoSlugField"])
 
-# ----- south signals handling -----
+
+# ----- signals handling -----
 
 post_save_receivers = None
 cache_backend = None
 
 
-def handle_pre_migrate(sender, **kwargs):
+def pre_migrate_handler(sender, **kwargs):
     global post_save_receivers, cache_backend
     # remove post save receivers because we cannot to call create_menus signal
     post_save_receivers = post_save.receivers
     post_save.receivers = []
     # use dummy cache backend because Johnny cache does weird things. See #852
-    cache_backend = settings.CACHE_BACKEND
-    settings.CACHE_BACKEND = 'dummy://'
+    cache_backend = settings.CACHES['default']['BACKEND']
+    settings.CACHES['default']['BACKEND'] = 'django.core.cache.backends.dummy.DummyCache'
 
 
-def handle_post_migrate(sender, **kwargs):
+def post_migrate_handler(sender, **kwargs):
     from merengue.pluggable import enable_active_plugins
     global post_save_receivers, cache_backend
     app = kwargs['app']
@@ -828,8 +870,58 @@ def handle_post_migrate(sender, **kwargs):
                 call_command('loaddata', fixture, verbosity=1)
     # will set again saved receivers and cache backend
     post_save.receivers = post_save_receivers
-    settings.CACHE_BACKEND = cache_backend
+    settings.CACHES['default']['BACKEND'] = cache_backend
 
 
-pre_migrate.connect(handle_pre_migrate)
-post_migrate.connect(handle_post_migrate)
+def notify_status_changes(sender, instance, **kwargs):
+    if isinstance(instance, BaseContent):
+        if not hasattr(instance, '_original_status'):
+            return
+        if instance._original_status == instance.status:
+            return
+        if instance.status == 'pending':
+            review_to_pending_status(instance, instance._original_status)
+        # Change cached original status so if we have
+        # multiple saves we don't generate multiple
+        # review tasks
+        instance._original_status = instance.status
+
+
+def calculate_class_name(instance):
+    instance.class_name = instance._meta.module_name
+
+
+def base_content_pre_save_handler(sender, instance, **kwargs):
+    if isinstance(instance, BaseContent) and not instance.id:
+        calculate_class_name(instance)
+
+
+def update_permission_handler(sender, instance, created, **kwargs):
+    if Base in instance.__class__.mro():
+        instance.populate_workflow_status()
+
+
+signals.pre_save.connect(base_content_pre_save_handler)
+signals.post_save.connect(update_permission_handler)
+signals.post_save.connect(notify_status_changes)
+
+pre_migrate.connect(pre_migrate_handler)
+post_migrate.connect(post_migrate_handler)
+
+max_file_size = getattr(settings, 'MERENGUE_MAX_FILE_SIZE', None)
+max_image_size = getattr(settings, 'MERENGUE_MAX_IMAGE_SIZE', None)
+
+if max_file_size or max_image_size:
+    from django.forms.fields import FileField, ImageField, Field
+
+    class MerengueFileField(Field):
+
+        def clean(self, data):
+            max_size = getattr(self, 'merengue_max_file_size', None)
+            if data and max_size and len(data) > max_size:
+                raise ValidationError(_('File to large. Max size restricted to %s bytes') % max_size)
+            return super(MerengueFileField, self).clean(data)
+
+    FileField.__bases__ = (MerengueFileField, )
+    FileField.merengue_max_file_size = max_file_size
+    ImageField.merengue_max_file_size = max_image_size or max_file_size
