@@ -23,9 +23,8 @@
             }
 
             var configureShadowBox = function() {
-                Shadowbox.setup($('.MultimediaItemThumbnail a'), {
-                    autoplayMovies: true,
-                    flvPlayer: mediaurl + 'merengue/flash/flvplayer.swf'
+                Shadowbox.setup(slide.find('.MultimediaItemThumbnail a'), {
+                    flvPlayer: mediaurl + 'merengue/flash/flowplayer-3.2.7.swf'
                 });
             }
 
@@ -76,7 +75,7 @@
             var addNewItem = function(typename, item) {
                 type = getType(typename);
                 type.items.push(item);
-                if ($(item).find('.MultimediaItemThumbnail a').attr('rel').indexOf('type=')>=0) {
+                if ($(item).find('.MultimediaItemThumbnail a').attr('rel').indexOf('video')>=0) {
                     $(item).find('.MultimediaItemThumbnail a').append('<span class="playbutton">&nbsp;</span>');
                 }
             };
