@@ -1,4 +1,4 @@
-# Copyright (c) 2010 by Yaco Sistemas <msaelices@yaco.es>
+# Copyright (c) 2010 by Yaco Sistemas
 #
 # This file is part of Merengue.
 #
@@ -16,13 +16,23 @@
 # along with Merengue.  If not, see <http://www.gnu.org/licenses/>.
 
 from django.conf import settings
+from django.core.exceptions import PermissionDenied
 from django.core.urlresolvers import reverse
 from django.template import RequestContext
 from django.template.loader import render_to_string
 from django.utils import translation
 
+from merengue.perms.views import access_request
 
-class HttpStatusCodeRendererMiddleware(object):
+
+class HttpStatusCode403Provider(object):
+
+    def process_exception(self, request, exception):
+        if isinstance(exception, PermissionDenied):
+            return access_request(request, exception)
+
+
+class HttpStatusCodeRendererMiddleware(HttpStatusCode403Provider):
     """This middleware render a template when an error code (not 500) is found"""
 
     def process_response(self, request, response):
@@ -49,7 +59,7 @@ class LocaleMiddleware(object):
         forced_lang = request.GET.get('set_language', None)
         if forced_lang:
             translation.activate(forced_lang)
-            request.LANGUAGE_CODE = translation.get_language()
+        request.LANGUAGE_CODE = translation.get_language()
 
     def process_response(self, request, response):
         forced_lang = request.GET.get('set_language', None)
@@ -106,8 +116,9 @@ class RemoveRandomAjaxParameter(object):
 def get_context(request):
     admin_prefix = reverse('admin_index')
     is_admin = request.get_full_path().startswith(admin_prefix)
-    # TODO: refactor to avoid hardcode "base.html" and "admin/base.html"
-    template_base = 'base.html'
-    if is_admin:
-        template_base = 'admin/%s' % template_base
+    # TODO: refactor to avoid hardcode "base.html" and "admin/base_site.html"
+    if not is_admin:
+        template_base = 'base.html'
+    else:
+        template_base = 'admin/base_site.html'
     return {'template_base': template_base, 'is_admin': is_admin}
