@@ -1,10 +1,29 @@
+# Copyright (c) 2010 by Yaco Sistemas
+#
+# This file is part of Merengue.
+#
+# Merengue is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# Merengue is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with Merengue.  If not, see <http://www.gnu.org/licenses/>.
+
 from django.db import models
+from django.db.models.signals import post_save, post_delete
 from django.contrib.auth.models import User
 from django.contrib.auth.models import Group
 from django.contrib.contenttypes.models import ContentType
 from django.utils.translation import ugettext_lazy as _
 
 from merengue.base.models import BaseContent
+from merengue.perms.managers import RoleManager
 
 
 class Permission(models.Model):
@@ -36,7 +55,7 @@ class Permission(models.Model):
         ordering = ('name', )
 
     def __unicode__(self):
-        return "%s (%s)" % (self.name, self.codename)
+        return u"%s (%s)" % (self.name, self.codename)
 
 
 class ObjectPermission(models.Model):
@@ -58,8 +77,11 @@ class ObjectPermission(models.Model):
 
     content = models.ForeignKey(BaseContent, blank=True, null=True)
 
+    class Meta:
+        unique_together = ('role', 'permission', 'content')
+
     def __unicode__(self):
-        return "%s / %s / %s" % (self.permission.name, self.role, self.content)
+        return u"%s / %s / %s" % (self.permission.name, self.role, self.content)
 
     def get_principal(self):
         """Returns the principal.
@@ -92,7 +114,7 @@ class ObjectPermissionInheritanceBlock(models.Model):
     content = models.ForeignKey(BaseContent)
 
     def __unicode__(self):
-        return "%s / %s" % (self.permission, self.content)
+        return u"%s / %s" % (self.permission, self.content)
 
 
 class Role(models.Model):
@@ -105,6 +127,9 @@ class Role(models.Model):
         The unique name of the role
     """
     name = models.CharField(max_length=100, unique=True)
+    slug = models.SlugField(verbose_name=_('slug'), max_length=200,
+                            db_index=True, unique=True)
+    objects = RoleManager()
 
     class Meta:
         ordering = ("name", )
@@ -171,6 +196,42 @@ class PrincipalRoleRelation(models.Model):
 
     def __unicode__(self):
         if self.content:
-            return "%s / %s / %s" % (self.user or self.group, self.role, self.content)
+            return u"%s / %s / %s" % (self.user or self.group, self.role, self.content)
         else:
-            return "%s / %s" % (self.user or self.group, self.role)
+            return u"%s / %s" % (self.user or self.group, self.role)
+
+
+class AccessRequest(models.Model):
+
+    url = models.CharField(verbose_name=_(u"URL"), max_length=250)
+    access_time = models.DateTimeField(verbose_name=_(u"Access Time"))
+    content = models.ForeignKey(BaseContent, verbose_name=_(u"content"), blank=True, null=True)
+    state = models.ForeignKey('workflow.State', verbose_name=_(u"State"),
+                              blank=True, null=True,
+                              help_text=_('State of the content in that moment of access request'))
+    user = models.ForeignKey(User, verbose_name=_(u"User"), blank=True, null=True)
+    permission = models.ForeignKey(Permission, verbose_name=_(u"Permission requested"),
+                                   blank=True, null=True,
+                                   help_text=_('Permission requested by the user'))
+    roles = models.ManyToManyField(Role, verbose_name=_(u"Roles"), blank=True, null=True,
+                                   help_text=_('Roles of the user in the moment of access request, for this content'))
+    request_notes = models.TextField(verbose_name=_('Request notes'),
+                                     null=True, blank=True)
+    owners = models.ManyToManyField(User, verbose_name=_(u"Owners"), blank=True, null=True, related_name='access_review')
+    is_done = models.BooleanField(verbose_name=_(u'is done'), default=False)
+
+    def __unicode__(self):
+        if self.content:
+            return unicode(self.content)
+        return self.url
+
+
+def clear_permission_cache_handler(sender, instance, **kwargs):
+    from merengue.perms.utils import clear_cache
+    clear_cache()
+
+
+post_save.connect(clear_permission_cache_handler, sender=ObjectPermission)
+post_save.connect(clear_permission_cache_handler, sender=PrincipalRoleRelation)
+post_delete.connect(clear_permission_cache_handler, sender=ObjectPermission)
+post_delete.connect(clear_permission_cache_handler, sender=PrincipalRoleRelation)
