@@ -30,6 +30,7 @@ from merengue.utils import is_last_application, classproperty
 class Plugin(RegistrableItem):
     singleton = True
     url_prefixes = ()
+    active_by_default = False
 
     def get_url_prefixes(self):
         prefixes = []
@@ -55,7 +56,7 @@ class Plugin(RegistrableItem):
 
     def _get_registered_items(self, item_classes):
         for item_class in item_classes:
-            reg_item = RegisteredItem.objects.by_item_class(item_class).get()
+            reg_item = RegisteredItem.objects.get_by_item_class(item_class)
             yield reg_item.get_registry_item()
 
     def get_actions(self):
@@ -76,6 +77,9 @@ class Plugin(RegistrableItem):
     def post_actions(self):
         pass
 
+    def models(self):
+        return []  # to override in plugins
+
     def section_models(self):
         return []  # to override in plugins
 
@@ -117,7 +121,7 @@ def register_plugin(plugin_dir):
     if plugin_config:
         validate_plugin(plugin_config)
         try:
-            reg_plugin = RegisteredPlugin.objects.by_item_class(plugin_config).get()
+            reg_plugin = RegisteredPlugin.objects.get_by_item_class(plugin_config)
         except RegisteredPlugin.DoesNotExist:
             reg_plugin = register(plugin_config)
         plugin = reg_plugin.get_registry_item()
@@ -173,21 +177,32 @@ def enable_active_plugins():
 
 
 def register_all_plugins(verbose=False):
-    from merengue.pluggable.utils import get_plugin_directories, get_plugin_config
-    for plugin_dir in get_plugin_directories():
-        try:
-            if verbose:
-                plugin_config = get_plugin_config(plugin_dir)
-                if plugin_config:
-                    if not have_registered_items(plugin_config):
-                        print 'Registering new plugin %s...' % plugin_dir
+    from merengue.pluggable.models import RegisteredPlugin
+    from merengue.pluggable.utils import (get_plugin_directories, get_plugin_config,
+                                          reload_models_cache, remove_from_installed_apps,
+                                          clear_plugin_module_cache, get_plugin_module_name)
+    from merengue.pluggable.checker import mark_broken_plugin
+    try:
+        for plugin_dir in get_plugin_directories():
+            try:
+                if verbose:
+                    plugin_config = get_plugin_config(plugin_dir)
+                    if plugin_config:
+                        if not have_registered_items(plugin_config):
+                            print 'Registering new plugin %s...' % plugin_dir
+                        else:
+                            print 'Re-registering plugin %s...' % plugin_dir
                     else:
-                        print 'Re-registering plugin %s...' % plugin_dir
-                else:
-                    print 'Error walking to plugin %s.' % plugin_dir
-            register_plugin(plugin_dir)
-        except:
-            print 'Error registering %s plugin... go to next plugin.' % plugin_dir
+                        print 'Error walking to plugin %s.' % plugin_dir
+                register_plugin(plugin_dir)
+            except:
+                mark_broken_plugin(plugin_dir)
+                print 'Error registering %s plugin... go to next plugin.' % plugin_dir
+    finally:
+        for plugin in RegisteredPlugin.objects.inactives():
+            clear_plugin_module_cache(get_plugin_module_name(plugin.directory_name))
+            remove_from_installed_apps(plugin.directory_name)
+        reload_models_cache()
 
 
 def active_default_plugins(*args, **kwargs):
@@ -199,17 +214,18 @@ def active_default_plugins(*args, **kwargs):
         # register required plugins
         for plugin_dir in settings.REQUIRED_PLUGINS:
             active_plugin_with_deps(plugin_dir)
-            from merengue.section.models import Menu
-            name_attr = get_fallback_fieldname('name')
-            attrs = {name_attr: 'Portal menu', 'slug': settings.MENU_PORTAL_SLUG}
-            try:
-                portal_menu = Menu.objects.get(slug=settings.MENU_PORTAL_SLUG)
-            except Menu.DoesNotExist:
-                # creating portal menu if does not exist
-                portal_menu = Menu.objects.create(**attrs)
-                for lang_code, lang_text in settings.LANGUAGES:
-                    setattr(portal_menu, 'name_%s' % lang_code, ugettext('portal menu'))
-                portal_menu.save()
+        # populate menu
+        from merengue.section.models import Menu
+        name_attr = get_fallback_fieldname('name')
+        attrs = {name_attr: 'Portal menu', 'slug': settings.MENU_PORTAL_SLUG}
+        try:
+            portal_menu = Menu.objects.get(slug=settings.MENU_PORTAL_SLUG)
+        except Menu.DoesNotExist:
+            # creating portal menu if does not exist
+            portal_menu = Menu.objects.create(**attrs)
+            for lang_code, lang_text in settings.LANGUAGES:
+                setattr(portal_menu, 'name_%s' % lang_code, ugettext('portal menu'))
+            portal_menu.save()
 
 
 def active_plugin_with_deps(plugin_dir):
