@@ -23,7 +23,7 @@ from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes import generic
 from django.utils.translation import ugettext_lazy as _
 
-from merengue.review.utils import send_mail_content_as_pending
+from merengue.review.utils import send_mail_content_as_pending, get_reviewers
 
 
 class ReviewTask(models.Model):
@@ -41,7 +41,7 @@ class ReviewTask(models.Model):
     task_object = generic.GenericForeignKey('task_object_type', 'task_object_id')
 
     class Meta:
-        ordering = ('is_done', 'title', 'url', 'owner')
+        ordering = ('is_done', 'title', 'url', 'owner', )
 
     def __unicode__(self):
         if self.is_done:
@@ -52,11 +52,14 @@ class ReviewTask(models.Model):
 
 
 def notify_review_task(sender, instance, created, **kwargs):
-    if getattr(settings, 'SEND_MAIL_IF_PENDING', False):
-        if created and not ReviewTask.objects.filter(
+    if created and getattr(settings, 'SEND_MAIL_IF_PENDING', False):
+        if not ReviewTask.objects.filter(
             is_done=False, task_object_id=instance.task_object_id).count() > 1:
-            assigned = [user for user in instance.assigned_to.all()]
-            send_mail_content_as_pending(instance.task_object,
+            # disclaimer: the assigned users are not selected using instance
+            # because they're not updated, because M2M fields are updated
+            # after the post_save signal
+            assigned = get_reviewers(instance.task_object)
+            send_mail_content_as_pending(instance.task_object, instance,
                                          [instance.owner] + assigned)
 
 
