@@ -3,7 +3,7 @@
 # Django settings for merengue project.
 
 from os import path
-from merengue.settings import *
+from merengue.settings import *  # pyflakes:ignore
 
 ugettext = lambda s: s  # dummy ugettext function, as said on django docs
 
@@ -21,17 +21,22 @@ ADMINS = (
 
 MANAGERS = ADMINS
 
-DATABASE_ENGINE = 'postgresql_psycopg2'           # 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
-DATABASE_NAME = ''             # Or path to database file if using sqlite3.
-DATABASE_USER = ''             # Not used with sqlite3.
-DATABASE_PASSWORD = ''         # Not used with sqlite3.
-DATABASE_HOST = ''             # Set to empty string for localhost. Not used with sqlite3.
-DATABASE_PORT = ''             # Set to empty string for default. Not used with sqlite3.
-
-if DATABASE_ENGINE == 'postgresql_psycopg2':
-    DATABASE_OPTIONS = {
-        'autocommit': True,
+DATABASES = {
+    'default': {
+        'ENGINE': 'django.db.backends.',  # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
+        'NAME': '',                       # Or path to database file if using sqlite3.
+        'USER': '',                       # Not used with sqlite3.
+        'PASSWORD': '',                   # Not used with sqlite3.
+        'HOST': '',                       # Set to empty string for localhost. Not used with sqlite3.
+        'PORT': '',                       # Set to empty string for default. Not used with sqlite3.
     }
+}
+
+for database in DATABASES.values():
+    if database['ENGINE'] == 'django.db.backends.postgresql_psycopg2':
+        database['OPTIONS'] = {
+            'autocommit': True,
+        }
 
 # Local time zone for this installation. Choices can be found here:
 # http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
@@ -47,7 +52,7 @@ LANGUAGE_CODE = 'en'
 LANGUAGES = (
     ('en', ugettext('English')),
     ('es', ugettext('Español')),
-    ('fr', ugettext('Français')),
+    ('it', ugettext('Italiano')),
 )
 
 URL_DEFAULT_LANG = LANGUAGE_CODE
@@ -58,13 +63,20 @@ SITE_ID = 1
 # to load the internationalization machinery.
 USE_I18N = True
 
+# Localization
+USE_L10N = True
+FORMAT_MODULE_PATH = 'formats'
+
 # PostGIS extension flag
 USE_GIS = False
 
 if USE_GIS:
     INSTALLED_APPS += ('django.contrib.gis', 'merengue.places', )
-    if DATABASE_ENGINE == 'postgresql_psycopg2':
+    if DATABASES['default']['ENGINE'] == 'django.db.backends.postgresql_psycopg2':
+        DATABASES['default']['ENGINE'] = 'django.contrib.gis.db.backends.postgis'
         TEST_DB_CREATION_SUFFIX = 'WITH TEMPLATE template_postgis'
+    if DATABASES['default']['ENGINE'] == 'django.db.backends.mysql':
+        DATABASES['default']['ENGINE'] = 'django.contrib.gis.db.backends.mysql'
 
 # Absolute path to the directory that holds media.
 # Example: "/home/media/media.lawrence.com/"
@@ -72,7 +84,7 @@ MEDIA_ROOT = path.join(BASEDIR, 'media/')
 
 # URL that handles the media served from MEDIA_ROOT. Make sure to use a
 # trailing slash if there is a path component (optional in other cases).
-# Examples: "http://media.lawrence.com", "http://example.com/meencodingdia/"
+# Examples: "http://media.lawrence.com", "http://example.com/media/"
 MEDIA_URL = '/media/'
 
 # URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
@@ -83,6 +95,9 @@ ADMIN_MEDIA_PREFIX = '/admin_media/'
 # Make this unique, and don't share it with anybody.
 SECRET_KEY = ''
 
+# Set to True for enabling site caching for anonymous visitors
+CACHE_SITE_FOR_ANONYMOUS = False
+
 MIDDLEWARE_CLASSES = PRE_MERENGUE_MIDDLEWARE_CLASSES + MERENGUE_MIDDLEWARE_CLASSES + POST_MERENGUE_MIDDLEWARE_CLASSES + (
     # put here aditional middlewares
 )
@@ -93,16 +108,12 @@ TEMPLATE_DIRS = (
     path.join(BASEDIR, 'templates'),
 ) + TEMPLATE_DIRS
 
-LOGIN_REDIRECT_URL = '/'
-LOGIN_URL = '/account/login/'
 
 INSTALLED_APPS += (
     # put here your project django apps
     'website',
 )
 
-TEST_DB_CREATION_SUFFIX = 'WITH TEMPLATE template_postgis'
-
 FIXTURE_DIRS = (
     path.join(BASEDIR, 'fixtures', ),
 )
@@ -129,11 +140,19 @@ PRODUCTION_DB_URL = ""
 PRODUCTION_DB_UPDATE_PASSWORDS = (('admin', 'admin'), )
 
 # For johnny cache. Johnny cache key prefix should not be the same in other projects
-CACHE_BACKEND = 'johnny.backends.locmem:///'
-JOHNNY_MIDDLEWARE_KEY_PREFIX = '%s-cache' % DATABASE_NAME
+CACHES = {
+    'default': {
+        'BACKEND': 'johnny.backends.locmem.LocMemCache',
+        'KEY_PREFIX': SECRET_KEY,
+        'OPTIONS': {
+            'MAX_ENTRIES': 3000,
+        },
+    }
+}
+JOHNNY_MIDDLEWARE_KEY_PREFIX = '%s-cache' % DATABASES['default']['NAME']
 
 # if merengue will detect new plugins in file system
-DETECT_NEW_PLUGINS = True
+DETECT_NEW_PLUGINS = False
 # if merengue will detect broken plugins
 DETECT_BROKEN_PLUGINS = True
 
@@ -149,8 +168,18 @@ DEBUG_TOOLBAR_PANELS = (
     #'debug_toolbar.panels.cache.CacheDebugPanel',
     'debug_toolbar.panels.template.TemplateDebugPanel',
     #'debug_toolbar.panels.logger.LoggingPanel',
+    'merengue.dbtoolbar.BlockDebugPanel',
 )
 
+# File maximum size
+# You can restrict the maximum size of uploaded files using
+# MERENGUE_MAX_FILE_SIZE for any type of file or
+# MERENGUE_MAX_IMAGE_SIZE for image files.
+# You can use one, both or neither
+#
+#MERENGUE_MAX_FILE_SIZE = 10485760  # 10Mb
+#MERENGUE_MAX_IMAGE_SIZE = 5242880  # 5Mb
+
 # Allow overwriting any configuration in optional settings_local.py
 # (it can be used to set up your own database, debug and cache options, contact mails...)
 try:
