@@ -16,12 +16,16 @@
 # along with Merengue.  If not, see <http://www.gnu.org/licenses/>.
 
 import sorl
-from merengue.base.admin import BaseContentAdmin, PluginAdmin
+from merengue.base.admin import BaseCategoryAdmin, BaseContentAdmin
 from merengue.section.admin import SectionContentAdmin
 from plugins.banner.models import Banner
 
 
-class BannerAdmin(PluginAdmin, BaseContentAdmin):
+class BannerCategoryAdmin(BaseCategoryAdmin):
+    """ Admin for news item category management """
+
+
+class BannerAdmin(BaseContentAdmin):
 
     def save_model(self, request, obj, form, change):
         saved = super(BannerAdmin, self).save_model(request, obj, form, change)
