@@ -1,4 +1,4 @@
-# Copyright (c) 2010 by Yaco Sistemas <msaelices@yaco.es>
+# Copyright (c) 2010 by Yaco Sistemas
 #
 # This file is part of Merengue.
 #
@@ -15,7 +15,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Merengue.  If not, see <http://www.gnu.org/licenses/>.
 
-from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext as _, ugettext_lazy
 
 from merengue.block.blocks import Block
 from plugins.chunks.models import Chunk
@@ -24,10 +24,11 @@ from plugins.chunks.models import Chunk
 class ChunksBlock(Block):
     name = 'chunksblock'
     default_place = 'all'
+    help_text = ugettext_lazy('Chunks Block')
+    verbose_name = ugettext_lazy('Chunks Block')
 
-    @classmethod
-    def render(cls, request, place, context, *args, **kwargs):
+    def render(self, request, place, context, *args, **kwargs):
         chunks = Chunk.objects.placed_at(place, request.get_full_path())
-        return cls.render_block(request, template_name='chunks/chunks_block.html',
-                                block_title=_('Chunks'),
-                                context={'chunks': chunks})
+        return self.render_block(request, template_name='chunks/chunks_block.html',
+                                 block_title=_('Chunks'),
+                                 context={'chunks': chunks})
