@@ -35,16 +35,18 @@ class PluginConfig(Plugin):
           'plugins.event.urls'),
     )
 
+    def models(self):
+        return [
+            (Event, EventAdmin),
+            (Category, EventCategoryAdmin),
+        ]
+
     def section_models(self):
         return [(Event, EventSectionAdmin)]
 
     def get_blocks(self):
         return [EventsCalendarBlock]
 
-    def get_model_admins(self):
-        return [(Event, EventAdmin),
-                (Category, EventCategoryAdmin)]
-
     def get_viewlets(self):
         return [LatestEventViewlet, AllEventViewlet]
 
@@ -52,8 +54,3 @@ class PluginConfig(Plugin):
         collection = create_normalize_collection('event', u'Event', Event,
                                                  create_display_field=True,
                                                  create_filter_field=True)
-        IncludeCollectionFilter.objects.get_or_create(
-            collection=collection,
-            filter_field='end',
-            filter_operator='date_gt',
-            )  # this way, we won't have in count the expired news
