@@ -2,8 +2,10 @@ import datetime
 
 from django.contrib.auth.models import User
 from django.contrib.contenttypes.models import ContentType
+from django.core.exceptions import ValidationError
 from django.db import models
 from django.db.models import permalink
+from django.utils.translation import ugettext
 from django.utils.translation import ugettext_lazy as _
 
 from merengue.base.models import BaseContent, BaseCategory
@@ -19,7 +21,8 @@ class ForumCategory(BaseCategory):
 
 class Forum(BaseContent):
 
-    category = models.ForeignKey(ForumCategory)
+    category = models.ForeignKey(ForumCategory,
+                                 verbose_name=_('Category'))
 
     objects = BaseContentManager()
 
@@ -28,6 +31,7 @@ class Forum(BaseContent):
         verbose_name_plural = _('Forums')
         content_view_function = 'plugins.forum.views.content_forum_view'
         content_view_template = 'forum/forum_view.html'
+        check_slug_uniqueness = True
 
     def _public_link_without_section(self):
         return ('forum_view', [self.slug])
@@ -64,14 +68,18 @@ class Thread(BaseContent):
     def get_main_section(self):
         return self.forum.get_main_section()
 
-    @permalink
-    def public_link(self):
+    def _public_link_without_section(self):
         return ('thread_view', [self.forum.slug, self.slug])
 
     def save(self, *args, **kwargs):
         self.commentable = 'disabled'
         super(Thread, self).save(*args, **kwargs)
 
+    def validate_unique(self, exclude=None):
+        super(Thread, self).validate_unique(exclude)
+        if self.forum_id and self.forum.thread_set.filter(slug=self.slug).exclude(pk=self.pk).exists():
+            raise ValidationError({'slug': (ugettext('The slug already exists in other thread in this forum'),)})
+
     def get_last_comment(self):
         comments = ForumThreadComment.objects.filter(thread=self).order_by('-date_submitted')
         return comments and comments[0] or None
