@@ -1,49 +1,57 @@
-from django.contrib.auth.decorators import login_required
-from django.core.urlresolvers import reverse
+import transmeta
+
 from django.http import HttpResponse, HttpResponseRedirect, Http404
 from django.shortcuts import get_object_or_404, render_to_response
 from django.template import RequestContext
-from django.template.defaultfilters import slugify
 from django.utils import simplejson
+from django.utils.translation import get_language
 from django.utils.translation import ugettext as _
 
 from captcha.decorators import add_captcha
 
+from merengue.base.decorators import login_required
+from merengue.base.utils import get_unique_slug
 from merengue.base.views import content_list, content_view, render_content
 from merengue.perms.utils import has_permission
 from merengue.section.utils import get_section, filtering_in_section
 
 from plugins.forum.models import Forum, Thread, ForumThreadComment
 from plugins.forum.forms import CaptchaForumThreadCommentForm, CreateThreadForm
+from plugins.forum.utils import can_create_new_thread
 
 
 PAGINATE_BY = 20
 
 
-def forum_index(request, extra_context={}):
+def forum_index(request, extra_context=None):
     section = get_section(request, extra_context)
-    forum_list = Forum.objects.published()
+    forum_list = Forum.objects.published().order_by(transmeta.get_real_fieldname('category__name', get_language()),
+                                                    transmeta.get_real_fieldname('name', get_language()))
     forum_list = filtering_in_section(forum_list, section)
     return content_list(request, forum_list, template_name='forum/forum_list.html', paginate_by=PAGINATE_BY)
 
 
-def forum_view(request, forum_slug, original_context=None):
+def forum_view(request, forum_slug, extra_context=None):
     forum = get_object_or_404(Forum, slug=forum_slug)
+    extra_context = extra_context or {}
     context = {'thread_list': forum.thread_set.published(),
                'paginate_threads_by': PAGINATE_BY,
               }
+    context.update(extra_context)
     return content_view(request, forum, extra_context=context)
 
 
-def content_forum_view(request, content, template_name, extra_context):
+def content_forum_view(request, content, template_name, extra_context=None):
+    extra_context = extra_context or {}
     context = {'thread_list': content.thread_set.published(),
                'paginate_threads_by': PAGINATE_BY,
               }
+    context.update(extra_context)
     return render_content(request, content, template_name, context)
 
 
 def thread_view(request, forum_slug, thread_slug, original_context=None):
-    thread = get_object_or_404(Thread, slug=thread_slug)
+    thread = get_object_or_404(Thread, slug=thread_slug, forum__slug=forum_slug)
     is_moderated = request.user and (request.user.is_superuser or has_permission(thread.forum, request.user, 'moderate_forum'))
     is_auth = request.user and request.user.is_authenticated()
     comments = thread.forumthreadcomment_set.filter(parent__isnull=True).order_by('date_submitted')
@@ -55,22 +63,25 @@ def thread_view(request, forum_slug, thread_slug, original_context=None):
 
 def create_new_thread(request, forum_slug):
     forum = get_object_or_404(Forum, slug=forum_slug)
-    if not request.user or not (request.user.is_superuser or has_permission(forum, request.user, 'create_new_thread')):
-        login_url = '%s?next=%s' % (reverse('merengue_login'),
-                                    request.get_full_path())
-        return HttpResponseRedirect(login_url)
+    http_response = can_create_new_thread(request, forum)
+    if http_response:
+        return http_response
     if request.POST:
-        form = CreateThreadForm(request.POST)
+        form = CreateThreadForm(forum, request.POST)
         if form.is_valid():
             thread = form.save(commit=False)
-            thread.slug = slugify(thread.name)
+            thread.slug = get_unique_slug(thread.name, forum.thread_set.all())
             thread.forum = forum
             thread.user = request.user
             thread.status = 'published'
             thread.save()
+            workflow_status = thread.workflow_status.get_all_states().filter(slug='published')
+            if workflow_status:
+                thread.workflow_status = workflow_status[0]
+                thread.save()
             return HttpResponseRedirect(thread.get_absolute_url())
     else:
-        form = CreateThreadForm()
+        form = CreateThreadForm(forum)
 
     return render_to_response('forum/forum_create_thread.html', {
             'form': form, 'content': forum,
@@ -102,7 +113,7 @@ def forum_comment_form(request, content, parent_id, form=None, template='forum/f
 
 @login_required
 def forum_comment_add(request, forum_slug, thread_slug, parent_id=None):
-    thread = get_object_or_404(Thread, slug=thread_slug)
+    thread = get_object_or_404(Thread, slug=thread_slug, forum__slug=forum_slug)
     if thread.closed:
         raise Http404
 
