@@ -16,11 +16,8 @@
 # along with Merengue.  If not, see <http://www.gnu.org/licenses/>.
 
 from django.db import models
-from django.db.models import permalink
 from django.utils.translation import ugettext_lazy as _
 
-from cmsutils.cache import CachingManager
-
 from merengue.base.managers import BaseContentManager
 from merengue.base.models import BaseCategory, BaseContent
 
@@ -31,19 +28,20 @@ class LinkCategory(BaseCategory):
         verbose_name = _('link category')
         verbose_name_plural = _('link categories')
 
-    objects = CachingManager(cache_object_retrieval=True)
-
 
 class Link(BaseContent):
 
     url_link = models.URLField(verbose_name=_('Url Link'))
     portal_name = models.CharField(verbose_name=_('Portal name'), max_length=250, blank=True, null=True)
+    categories = models.ManyToManyField(LinkCategory,
+                                        verbose_name=_('category'),
+                                        blank=True, null=True, db_index=True)
     objects = BaseContentManager()
 
     class Meta:
         verbose_name = _('link')
         verbose_name_plural = _('links')
+        check_slug_uniqueness = True
 
-    @permalink
-    def public_link(self):
+    def _public_link_without_section(self):
         return ('link_view', [self.slug])
