@@ -28,10 +28,11 @@ class MicroSiteAdminForm(BaseAdminModelForm):
         # checked against direct parents, so MicroSite's slug could not
         # be unique among other BaseContents'
         cleaned_data = super(MicroSiteAdminForm, self).clean()
-        slug = self.cleaned_data['slug']
-        if MicroSite.objects.filter(slug__exact=slug).count() or \
-           BaseSection.objects.filter(slug__exact=slug).count():
-            invalid_slug = _('A MicroSite or Section with this slug already exists')
-            self._errors["slug"] = self.error_class([invalid_slug])
-            del cleaned_data['slug']
+        slug = self.cleaned_data.get('slug', None)
+        if slug and self.instance.id and slug != self.instance.slug:
+            if MicroSite.objects.filter(slug__exact=slug).count() or \
+                    BaseSection.objects.filter(slug__exact=slug).count():
+                invalid_slug = _('A MicroSite or Section with this slug already exists')
+                self._errors["slug"] = self.error_class([invalid_slug])
+                del cleaned_data['slug']
         return cleaned_data
