@@ -18,12 +18,9 @@
 import datetime
 
 from django.db import models
-from django.db.models import permalink
 from django.db.models.signals import pre_save
 from django.utils.translation import ugettext_lazy as _
 
-from cmsutils.cache import CachingManager
-
 from merengue.base.models import BaseContent, BaseCategory
 from plugins.news.managers import NewsItemManager
 
@@ -34,13 +31,17 @@ class NewsCategory(BaseCategory):
         verbose_name = _('news category')
         verbose_name_plural = _('news categories')
 
-    objects = CachingManager(cache_object_retrieval=True)
-
 
 class NewsItem(BaseContent):
 
-    publish_date = models.DateTimeField(blank=True, null=True, db_index=True)
-    expire_date = models.DateTimeField(blank=True, null=True, db_index=True)
+    publish_date = models.DateTimeField(verbose_name=_('publish date'),
+                                        blank=True,
+                                        null=True,
+                                        db_index=True)
+    expire_date = models.DateTimeField(verbose_name=_('expire date'),
+                                       blank=True,
+                                       null=True,
+                                       db_index=True)
     categories = models.ManyToManyField(NewsCategory,
                                         verbose_name=_('category'),
                                         blank=True, null=True, db_index=True)
@@ -52,15 +53,12 @@ class NewsItem(BaseContent):
         translate = ('body', )
         content_view_template = 'news/newsitem_view.html'
         ordering = ('-publish_date', '-id')
+        verbose_name = _('news item')
+        verbose_name_plural = _('news')
+        check_slug_uniqueness = True
 
-    @permalink
-    def public_link(self):
-        section = self.get_main_section()
-        if section is None:
-            return ('newsitem_view', [self.slug])
-        else:
-            # go to news item inside section which created it
-            return ('content_section_view', [section.slug, self.id, self.slug])
+    def _public_link_without_section(self):
+        return ('newsitem_view', [self.slug])
 
 
 def set_publish_date(sender, instance, **kwargs):
