@@ -19,21 +19,31 @@ from django.core.urlresolvers import reverse
 from django.utils.translation import ugettext as _, ugettext_lazy
 
 from merengue.block.blocks import Block
+from merengue.pluggable.utils import get_plugin
 
 
 class RSSGlobalFeed(Block):
     name = 'RSSGlobalFeed'
     default_place = 'meta'
+    fixed_place = True
+    is_addable = False
     help_text = ugettext_lazy('Block represents global RSS feed')
     verbose_name = ugettext_lazy('Global RSS feed block')
 
-    @classmethod
-    def render(cls, request, place, context, *args, **kwargs):
-        from plugins.rss.config import PluginConfig
-        portal_title = PluginConfig.get_config().get(
-            'portal', '').get_value()
+    default_caching_params = {
+        'enabled': True,
+        'timeout': 604800,
+        'only_anonymous': False,
+        'vary_on_user': False,
+        'vary_on_url': False,
+        'vary_on_language': False,
+    }
+
+    def render(self, request, place, context, *args, **kwargs):
+        plugin = get_plugin("rss")
+        portal_title = plugin.get_config().get('portal', '')
         feed_url = reverse('rss_views')
-        return cls.render_block(
+        return self.render_block(
             request, template_name='rss/global_block.html',
             block_title=_('RSS'),
             context={'feed_url': feed_url,
