@@ -1,4 +1,4 @@
-# Copyright (c) 2010 by Yaco Sistemas <msaelices@yaco.es>
+# Copyright (c) 2010 by Yaco Sistemas
 #
 # This file is part of Merengue.
 #
@@ -22,6 +22,9 @@ from django.utils.translation import ugettext_lazy as _
 
 from merengue.base.models import BaseCategory
 
+from transmeta import TransMeta
+from stdimage import StdImageField
+
 
 class StandingOutCategory(BaseCategory):
 
@@ -34,11 +37,28 @@ class StandingOutCategory(BaseCategory):
 
 class StandingOut(models.Model):
 
+    __metaclass__ = TransMeta
+
+    title = models.CharField(_('title'), max_length=200, blank=True, null=True)
+    short_description = models.TextField(_('short description'), blank=True, null=True)
+    image = StdImageField(
+        _('image'),
+        help_text=_('If you don\'t specify any image then the first photo (if any) associated to the content will be shown'),
+        upload_to='standingout',
+        thumbnail_size=(350, 240),
+        null=True,
+        blank=True,
+        )
     obj_content_type = models.ForeignKey(ContentType, verbose_name=_('obj content type'),
-                                         related_name='standingout_objects')
-    obj_id = models.PositiveIntegerField(_('object id'), db_index=True)
+                                         related_name='standingout_objects', null=True, blank=True)
+    obj_id = models.PositiveIntegerField(_('object id'), db_index=True, null=True, blank=True)
     obj = generic.GenericForeignKey('obj_content_type', 'obj_id')
 
+    url = models.CharField(
+        verbose_name=_('url'), max_length=200, null=True, blank=True,
+        help_text=_('The absolute urls have to write complety: Protocol, domain, query'),
+    )
+
     related_content_type = models.ForeignKey(ContentType, verbose_name=_('related content type'), null=True, blank=True,
                                              related_name='standingout_relateds')
     related_id = models.PositiveIntegerField(_('related object id'), db_index=True, null=True, blank=True)
@@ -50,13 +70,45 @@ class StandingOut(models.Model):
     order = models.IntegerField(_("Order"), blank=True, null=True)
 
     class Meta:
-        ordering = ('order', )
-        unique_together = (('obj_content_type', 'obj_id', 'related_content_type', 'related_id', 'standing_out_category'), )
+        ordering = ('-order', )
+        translate = ('title', 'short_description')
+        verbose_name = _('standing out')
+        verbose_name_plural = _('standing outs')
+
+    @property
+    def name(self):
+        return self.title or self.obj and unicode(self.obj) or self.target_name()
+
+    @property
+    def description(self):
+        return self.short_description or getattr(self.obj, 'description', '')
+
+    def has_description(self):
+        return True
+
+    @property
+    def main_image(self):
+        return self.obj.main_image
 
     def get_absolute_url(self):
-        return self.obj.get_absolute_url()
+        url = None
+        if hasattr(self.obj, 'get_absolute_url'):
+            url = self.obj.get_absolute_url()
+        else:
+            url = self.url
+        return url
+
+    def target_name(self):
+        target = ''
+        if self.obj:
+            target = unicode(self.obj)
+        elif self.url:
+            target = self.url
+        return target
 
     def __unicode__(self):
-        if not self.related_content_type or not self.related_id:
+        if self.title:
+            return self.title
+        elif self.obj:
             return unicode(self.obj)
-        return "%s --> %s" %(unicode(self.obj), unicode(self.related))
+        return 'without content'
